/*
 *
 *		wmmge dockapp for windowMaker
 *
 * Copyright (C) 2001 bennyben (benoit bourdin) 
 *   <bennybben@caramail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
	/* wmgeneral */
#include "../wmgeneral/wmgeneral.h"
	/* xlib */
#include <X11/Xlib.h>
#include <X11/xpm.h>
	/* pixmaps */
#include "wmmge-master.xpm"
#include "wmmge-mask.xbm"
#include "wmmge_master.h"
	/* wmmge */
#include "wmmge.h"

	/* global variables */
unsigned int wmmge_remaining_time;
unsigned int wmmge_clock;
bool wmmge_powerfail;


	/* constants */
const int wmmge_refresh_delay=	500000;		/* delay of 1/2 second */
const char wmmge_psp_dir[]=	"/usr/upsman/psp";

	/* tables */
const struct wmmge_events wmmge_event_table[]={
	{Expose,	OnExpose},
	{DestroyNotify,	OnDestroyNotify},
	{ButtonPress,	OnButtonPress},
	{ButtonRelease,	OnButtonRelease},
	{-1,		NULL}
};


int main(int argc, char **argv)
{
	openXwindow(argc, argv, wmmge_master_xpm, 
		wmmge_mask_bits, wmmge_mask_width, wmmge_mask_height);
	
	get_infos();
	refresh_display();
	usleep(500000);		/* wait a 1/2 second for events */
	for(wmmge_clock=0;;wmmge_clock++) {
		refresh_display();
		process_events();
		usleep(wmmge_refresh_delay);
		if( !(wmmge_clock%10) )
			get_infos();
		if(wmmge_clock>WMMGE_MAX_CLOCK)
			wmmge_clock=0;
	}
	return 0;
}

void process_events()
{
	XEvent          Event;
	
	while (XPending(display)) {
		XNextEvent(display, &Event);
		switch (Event.type) {
		case Expose:
			RedrawWindowXY(0, 0);
			break;
		case DestroyNotify:
			XCloseDisplay(display);
			exit(0);
			break;
		case ButtonPress:
			printf("button press ");
			break;
		case ButtonRelease:
			printf("button release ");
			break;
		}
	}
}

void refresh_display()
{
	disp_back(wmmge_remaining_time<WMMGE_ALERT_DELAY && wmmge_clock%2 );
	disp_remaining_time( wmmge_powerfail && wmmge_clock%2);
	disp_socket( !wmmge_powerfail || (wmmge_powerfail && !(wmmge_clock%2)) );
	RedrawWindowXY(0, 0);
}

void get_infos()
{
		/* get infos from the ups */
	char cmd[WMMGE_MAX_CMD];
	FILE *ftmp;
	unsigned int iL;
	char load[WMMGE_READ_MAX], status[WMMGE_READ_MAX];

printf("info\n");

		/* get status */
	snprintf(cmd, WMMGE_MAX_CMD, "%s/%s > %s", wmmge_psp_dir, WMMGE_GETSTATUS_CMD, WMMGE_TMP_STATUS_FILE);
	system(cmd);
		
	iL=0;
	ftmp=fopen(WMMGE_TMP_STATUS_FILE,"r");
	while(iL<WMMGE_READ_MAX-1 && !feof(ftmp) )
		fread(status+(iL++), 1, 1, ftmp);
	status[iL]='\0';
	fclose(ftmp);
	
		/* get load */
	snprintf(cmd, WMMGE_MAX_CMD, "%s/%s > %s", wmmge_psp_dir, WMMGE_GETLOAD_CMD, WMMGE_TMP_LOAD_FILE);
	system(cmd);
	
	iL=0;
	ftmp=fopen(WMMGE_TMP_LOAD_FILE,"r");
	while(iL<WMMGE_READ_MAX-1 && !feof(ftmp) )
		fread(load+(iL++), 1, 1, ftmp);
	load[iL]='\0';
	fclose(ftmp);
	
	wmmge_remaining_time=atoi(load);
	wmmge_powerfail=atoi(status);
}

/******************************* callbacks *****************************/

void OnExpose()
{

}

void OnDestroyNotify()
{

}

void OnButtonPress()
{

}

void OnButtonRelease()
{

}


/**************************** display functions ************************/

void disp_remaining_time(bool flash)
{
		/* displays the battery capacity (less than 99 minutes....) */
	int mn,s;
	
	mn=wmmge_remaining_time / 60;
	s=wmmge_remaining_time % 60;
	
	disp_figure(mn/10,
		WMMGE_FIGURES_DEST_X,
		WMMGE_FIGURES_DEST_Y
		,flash);
	disp_figure(mn%10,
		WMMGE_FIGURES_DEST_X+WMMGE_FIGURES_WIDTH+1,
		WMMGE_FIGURES_DEST_Y
		,flash);
	
	disp_figure(s/10,
		WMMGE_FIGURES_DEST_X+2*WMMGE_FIGURES_WIDTH+4,
		WMMGE_FIGURES_DEST_Y
		,flash);
	disp_figure(s%10,
		WMMGE_FIGURES_DEST_X+3*WMMGE_FIGURES_WIDTH+5,
		WMMGE_FIGURES_DEST_Y
		,flash);
}

void disp_back(bool flash)
{
		/* displays the baskground flashing or not */
	copyXPMArea( flash?WMMGE_BACK_SRC_FX:WMMGE_BACK_SRC_X,
		flash?WMMGE_BACK_SRC_FY:WMMGE_BACK_SRC_Y,
		WMMGE_BACK_WIDTH,
		WMMGE_BACK_HEIGHT,
		WMMGE_BACK_DEST_X,
		WMMGE_BACK_DEST_Y);
}

void disp_figure(unsigned int n,unsigned int x, unsigned int y, bool flash)
{
		/* displays a figure (0-9) at the wanted place */

	/* copyXPMArea args: src_x, src_y, width, height, dest_x, dest_y */
	copyXPMArea( flash?WMMGE_FIGURES_SRC_FX:WMMGE_FIGURES_SRC_X,
		(flash?WMMGE_FIGURES_SRC_FY:WMMGE_FIGURES_SRC_Y)
			+WMMGE_FIGURES_HEIGHT*n,
		WMMGE_FIGURES_WIDTH,
		WMMGE_FIGURES_HEIGHT,
		x,y);
}

void disp_socket(bool flash)
{
		/* displays the socket flashing or not */
	copyXPMArea( flash?WMMGE_SOCKET_SRC_FX:WMMGE_SOCKET_SRC_X,
		flash?WMMGE_SOCKET_SRC_FY:WMMGE_SOCKET_SRC_Y,
		WMMGE_SOCKET_WIDTH,
		WMMGE_SOCKET_HEIGHT,
		WMMGE_SOCKET_DEST_X-(!flash),
		WMMGE_SOCKET_DEST_Y);
	/* we must write width-1 to not have display problems */
}
